######### Supplemental material: Accuracy of thematic maps #########
#credits to Brian Costa at NOAA/NOS/NCCOS/CCMA for creating the embryo to this script

#example using HUB level 3, but can easily be modified to other classes

####Calculate AA statistics for Raster Predictions####

####load library####
library(maptools)
library (raster)
library(xlsx)
library(gdata) #load library that allows binding of df with unequal col/rows


#### setting paths ####
#forloop option if you have multiple model versions you want to test, activate if it is helpful for your setup
#for (i in c(x:x)){
i=1 #folder number since we had our outputs organized in folders with a version number
way = "path/substratecomponents/"
folder = paste("v0",i, sep='')
typeofmodel ="name"
path2rasteroutputs = paste(way,folder,"/", sep='')

path2aapoints = "path/points/"
path2aaoutputs = "path/aa/"
setwd("path") #set working directory

testdata = "testdata.csv"
path2testdata = paste(path2aapoints, testdata, sep='')
rasterpred = paste("hub3_",typeofmodel,"_v01.tif", sep='') #comp raster

path2rasterpred = paste(path2rasteroutputs, rasterpred, sep='')
path2aatable = paste(path2aaoutputs, "AAtable_", strsplit(rasterpred, ".tif"), ".xlsx", sep='')

#### Import AA pts, intersect with and extract values from habitat map ####

#import AA pts
aapts <- read.csv(paste(path2testdata, sep=''), header=T, sep=",")
aapts <- as.data.frame(aapts)

####import habitat map prediction####	
habmap <- raster(path2rasterpred)

#change column names if needed...
names(aapts)
colnames(aapts)[70]<-"HUB3" 
colnames(aapts)[71]<-"HUB4" 
colnames(aapts)[72]<-"HUB5" 
colnames(aapts)[73]<-"HUB6" 
colnames(aapts)[73]<-"HELCOM_HUB_CLASS" 

####separate table into different habitat classes####
aaclass <- subset(aapts, select=c(X_SWE99TM, Y_SWE99TM, HUB3)) #Choose aa class
coordinates(aaclass) = ~ X_SWE99TM + Y_SWE99TM #convert test data into spatial object

#extract values from predicted habitat map at AA locations
library(sp) #load library
o <- extract(habmap, aaclass, method="simple", sp=TRUE)

#view prediction and AA point locations

plot(habmap, col=topo.colors(100))#plot prediction
points(o) #overlay AA points
title(main = "HUB 3", font.main = 4)#add title

#format table
aaclass <- as.data.frame(o) #convert to data frame

aaclass <- na.omit(aaclass) #remove NA values
#row.names(aaclass) <- NULL #remove row names if exist

colnames(aaclass)[3]<-"CLASS_AA" #change column name
colnames(aaclass)[4]<-"CLASS_MAP" #change column name


aaclass$CLASS_AA <- as.character(aaclass$CLASS_AA)
aaclass$CLASS_MAP <- as.character(aaclass$CLASS_MAP)


####convert class numbers to class names ####
#letter code for HUB 3 classes used in the testdata.csv, changed to test string
aaclass[aaclass=="A"]<-"rock & boulder"
aaclass[aaclass=="B"]<-"hard clay"
aaclass[aaclass=="I"]<-"coarse substrate"
aaclass[aaclass=="J"]<-"sand"
aaclass[aaclass=="M"]<-"mixed"

####convert class numbers to class names ####
#numeric code was produced in the raster outputs instead of the original letter code, here we change it to a text string
aaclass[aaclass=="1"]<-"rock & boulder"
aaclass[aaclass=="2"]<-"hard clay"
aaclass[aaclass=="3"]<-"coarse substrate"
aaclass[aaclass=="4"]<-"sand"
aaclass[aaclass=="5"]<-"mixed"


#remove land, unknown, unclassified categories
aaclass[aaclass=="NA"] <- NA
aaclass[aaclass=="Unknown"] <- NA
aaclass <- aaclass[complete.cases(aaclass),] #remove NA values
aaclass$CLASS_AA <- factor(aaclass$CLASS_AA) #remove dimnames with NA
aaclass$CLASS_MAP <- factor(aaclass$CLASS_MAP) #remove dimnames with NA



#remove land, unknown, unclassified categories
aaclass[aaclass=="NA"] <- NA
aaclass[aaclass=="Unknown"] <- NA
aaclass <- aaclass[complete.cases(aaclass),] #remove NA values
aaclass$CLASS_AA <- factor(aaclass$CLASS_AA) #remove dimnames with NA
aaclass$CLASS_MAP <- factor(aaclass$CLASS_MAP) #remove dimnames with NA


#### Calculate frequencies to develop confusion matrix####
#calculate frequencies
AAresults_aaclass <- table(aaclass$CLASS_MAP, aaclass$CLASS_AA)
ft <- as.data.frame.matrix(AAresults_aaclass) #convert to dataframe to view

#check to see if rows match the columns
rnmsAA <-sort(unique(aaclass$CLASS_AA))
rnmsAA <- as.character(rnmsAA)
rmissing <- setdiff(rnmsAA, rownames(AAresults_aaclass)) # Find missing rows

numcol=NROW(rmissing)

numcol


####dont run if nrow missing = 0 ####
if(numcol<=0)
{
  print("no additional rows needed");
} else if (numcol>0){
  for (i in 1:numcol)
  {
    AAresults_aaclass = rbind(AAresults_aaclass, "NA"=0) 
    AAresults_aaclass <- as.data.frame(AAresults_aaclass)
    rownames(AAresults_aaclass)[which(rownames(AAresults_aaclass)=="NA")]= rmissing[i]
  }
}

#### start again ####

#sort rows by row name
AAresults_aaclass <- AAresults_aaclass[order(row.names(AAresults_aaclass)),]

#check to see if columns match the rows
cnmsMAP <-sort(unique(aaclass$CLASS_MAP))
cnmsMAP <- as.character(cnmsMAP)
cmissing <- setdiff(cnmsMAP, colnames(AAresults_aaclass)) # Find missing cols

numrow=NROW(cmissing)

numrow

####dont run if nrow missing = 0 ####

if(numrow<=0)
{
  print("no additional columns needed");
} else if (numrow>0){
  for (i in 1:numrow)
  {
    AAresults_aaclass = cbind(AAresults_aaclass, "NA"=0) 
    AAresults_aaclass <- as.data.frame(AAresults_aaclass)
    colnames(AAresults_aaclass)[which(colnames(AAresults_aaclass)=="NA")]= cmissing[i]
    
  }
}

#### start again ####


#sort columns by column name
AAresults_aaclass <- AAresults_aaclass[order(colnames(AAresults_aaclass)),]





####dont run if nrow missing = 0 ####
#update number of rows and columns

numrow=NROW(AAresults_aaclass)
numcol=NCOL(AAresults_aaclass)
AAresults_aaclass <- as.data.frame(AAresults_aaclass)
AAresults_aaclass <- as.matrix(AAresults_aaclass)

####start again####

###OA diagonal values####
dv_aaclass <- diag(AAresults_aaclass) #extract diagonal values for calc OA
dv_aaclass <- as.data.frame(sum(dv_aaclass)) #sum and convert to dataframe

#sum rows and columns
AAresults_aaclass <- addmargins(AAresults_aaclass)

#==================================================
##Calculate user's (UA) and producer's (PA) accuracy
diagval_aaclass <- diag(AAresults_aaclass) #extract diagonal values
diagval_aaclass <- as.data.frame(diagval_aaclass) #convert to dataframe

#calculate UA
UA_aaclass <- (diagval_aaclass/AAresults_aaclass[ ,"Sum"])*100
UA_aaclass <- unlist(UA_aaclass)
UA_aaclass <- as.numeric(UA_aaclass)

#calculate PA
PA_aaclass <- (diagval_aaclass/AAresults_aaclass["Sum", ])*100
PA_aaclass <- unlist(PA_aaclass)
PA_aaclass <- as.numeric(PA_aaclass)

#add UA and PA to confusion table
AAresults_aaclass = rbind(AAresults_aaclass, PA_aaclass) # add row for PA
rownames(AAresults_aaclass)[nrow=NROW(AAresults_aaclass)] <- "PA" 
AAresults_aaclass <- as.data.frame(AAresults_aaclass) #convert to df
UA_aaclass <- as.data.frame(UA_aaclass) #convert to df
AAresults_aaclass = cbindX(AAresults_aaclass, UA_aaclass) #add column for UA
colnames(AAresults_aaclass)[ncol=NCOL(AAresults_aaclass)] <- "UA" 

#get rid of NA values
AAresults_aaclass[is.na(AAresults_aaclass)] <- 0
AAresults_aaclass["Sum","UA"] = 0
AAresults_aaclass["PA","Sum"] = 0

##Calculate overall accuracy (OA)
Sum_aaclass = AAresults_aaclass["Sum","Sum"]
OA_aaclass = (dv_aaclass/Sum_aaclass)*100
AAresults_aaclass["PA","UA"] = OA_aaclass

numrow=NROW(AAresults_aaclass)-2

#Calculate tau, vtau, cltau
OA_aaclass=OA_aaclass/100
Pr_aaclass <- 1/numrow
tau_aaclass =((OA_aaclass-Pr_aaclass)/(1-Pr_aaclass))
vtau_aaclass = OA_aaclass*(1- OA_aaclass)/(Sum_aaclass*(1-Pr_aaclass)^2)
cltau_aaclass = 1.96*(vtau_aaclass^0.5)
tauall_aaclass <- matrix(c(tau_aaclass, vtau_aaclass, cltau_aaclass), 3)
rownames(tauall_aaclass) <- c("tau", "vtau", "cltau")

#==================================================
## Save AA results to .xlsx
# create work book
wb <- createWorkbook()

# create a sheet in the workbook for AA results
sheet <- createSheet(wb, sheetName="AA")
# add the data to the new sheet
addDataFrame(AAresults_aaclass, sheet)

AAresults_aaclass

# create a sheet in the workbook for tau results
sheet <- createSheet(wb, sheetName="TAU")
# add the data to the new sheet
addDataFrame(tauall_aaclass, sheet)

# save the workbook
saveWorkbook(wb, path2aatable)
