######### Supplemental material: model preparation #########

#### load packages ####
library(raster)
library(RStoolbox)
library(maptools)
library(caret) #wrapper for multiple types of modeling techniques
library(gbm) #package contains BRTs
library(rgdal) #package lets you I/O rasters
library(doParallel)
library(gbm)
library(e1071)
## set working directory

#### setting paths ####
way = "path"
setwd(way) ## set working directory
path2pred = "path/inputs/rasters/"
path2points = "path/inputs/points/"

#### import data table into R ####
dat_survey <- read.csv(paste(path2points, "annotations_point_intersept.csv", sep=''), header=T, sep=",") #high quality survey data to be used for both testing and training
dat_additional <- read.csv(paste(path2points, "annotations_legacy_expert.csv", sep=''), header=T, sep=",") #only used to train geological models (no testing), remove for biological models

#### test/train data partition ####
set.seed(28)  #Set seed so can reproduce same model trainingdataset for substrate and biology, important!

#Use 70% of data to train model
inTraining <- createDataPartition(dat_survey$HELCOM_HUB.Class, p = 0.70, list = FALSE) #using HUB level 1-6 reclassified in the trainingdata to partition the test/train data (HUB cathegories produced from percent coverage data using conditional statement in excel, but you can modify our r script to reclassify model outputs to HUB classes, to work on csv file as an alternative) 
training <- dat_survey[inTraining, ]
names(training)
#create final training dataset for geology parameters by combining training partition for hob16 and historical data
dat_traingeo <-merge(training, dat_additional, all = T)
training <- dat_traingeo

#Use remaining 30% of data to validate model later
testing <- dat_survey[-inTraining, ]

#convert test and train data into spatial objects
coordinates(training) = ~X_SWE99TM+Y_SWE99TM  #convert train data into spatial object
proj4string(training) <- CRS("+proj=utm +zone=33 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs") # Used projection is Sweref99 TM (UTM33N)  # EPSG:3006
writePointsShape(training, "traindata_geo")

coordinates(testing) = ~X_SWE99TM+Y_SWE99TM  #convert test data into spatial object
proj4string(testing) <- CRS("+proj=utm +zone=33 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs") # Used projection is Sweref99 TM (UTM33N)  # EPSG:3006
writePointsShape(testing, "testdata")

#### start forloop ####

#### stack environmental variables ####
setwd(path2pred)
env = stack(paste(path2pred, dir()[1], sep=''))
proj4string(env) <- CRS("+proj=utm +zone=33 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs") # Used projection is Sweref99 TM (UTM33N)  # EPSG:3006
  for (l in 2:length(dir())){
    env= stack( env, stack(paste(path2pred, dir()[l], sep=""))) }
rm(l)

#### spatial intersection points and env variables ####

df = training
df@data<- cbind(df@data, extract(env, df))
df<- as.data.frame(df)
names(df)


#### Choose your training variable ####
#set up a foreloop here if several models are to be run, it is not included in this script#


i= #choose your column number
TrainClasses <- df[,i]
TrainData <- df[,c(xx:xx)] #your interval of columns containing predictor variables


#### now continoue with the model script of choice####
#some modifications may have to be done to this script to fit each indivual model method
