
######### Supplemental material: continuous indivdual modelling #########
#this script can be used to run all substrate and biological models of continous data
#the model outputs may need to be adjusted to sum to 100% or whatever alse is a requirement for your classification scheme

####first run "2. model_peparation" script####
#version number, change!
v=1

colnames(df) 

####select your compositional models to be run using the forloop below####
for (i in c(xx:xx)) {
print(colnames(df)[i]) 
#### set env variables and train class####

TrainData <- df[,c(xx:xx)] #select the columns with your predictor variables

TrainClasses <- df[,i]


#### Tranformation ####
TrainClasses[TrainClasses == 0] <- 0.001    #remove -infinity (0% cover) and replace with 0.001%
TrainClasses[TrainClasses == 100] <- 99.999  #remove infinity (100% cover) and replace with 99.999%

TrainClasses_01 <- TrainClasses/100 #converting 0-100 to 0-1

TrainClasses_01log <- log(TrainClasses_01/(1 - TrainClasses_01))  #logit transformation to get gaussian distribution

hist.default(TrainClasses_01log) #plot historigram to check for normal distribution of transformed data

#### continuous boosted regression tree modelliing ####

#setup parallel processing for faster computing
nCPUs <- detectCores()-1
library(doParallel)
cl <- makeCluster(nCPUs)
registerDoParallel(cl)


#Optimize gbm input parameters, change to your prefered settings
gbmGrid <- expand.grid(.n.trees = c(1000, 1500),
                       .interaction.depth = c(10,15),
                       .shrinkage = c(0.01,0.005),
                       .n.minobsinnode = c(3))


#set your traingdata to the transformed numeric class
TrainClasses <- as.numeric(TrainClasses_01log)

####Develop gbm model###
rm(gbm.mod) #remove any legacy model loaded by mistake
gbm.mod <- train(TrainData, TrainClasses, method = "gbm", verbose = FALSE, distribution="gaussian", tuneGrid=gbmGrid, metric="RMSE", maximize=FALSE)

save("gbm.mod", file = paste("path",colnames(df)[i],"_version_",v,".RData", sep="")) #save model to RData file
rm("gbm.mod") #remove model from workspace
load(paste("path",colnames(df)[i],"_version_",v,".RData", sep="")) #load model back into workspace

print(gbm.mod)
print(summary(gbm.mod))


#### spatial prediction ####
rasters <- env


#apply gbm.mod model across space
rast.pred <- predict(rasters, gbm.mod, progress="text", type="raw")

#### backtransform raster prediction from logit transformation####
rast.pred.bt <- (exp(rast.pred)/(1+exp(rast.pred)))

stopCluster(cl)

#Export predictions to geotiff file for use as layer in GIS

writeRaster(rast.pred.bt, paste("path",colnames(df)[i],"_version_",v,".tif", sep=""), overwrite=TRUE, COMPRESS=LZW)

print(paste(colnames(df)[i],"_version_",v,".tif"," saved to file", sep=""))

}
####end foreloop####

#after this step you need to adjust your models to sum to 100% if that is a requirement. 
#one simple way to do that is to divide all compositional rasters with the sum of all rasters, 
#but you could also decide to let one component to remain unchanged and adjust the others if that makes more sense. 
#this step is not included in the supplemental material

