######### Supplemental material: reclassification from continuous models to hub3 thematic maps #########
#this script can also be modified to work on csv file to reclassify annotated percent coverage data# 

####load packages####

library(raster)
library(sp)
library(rgdal)
library(foreign)

#### setting paths ####
way = "path/"
folder="foldername"
typeofmodel ="name"
path2pred = paste(way,folder,"/", sep='')
path2output = "path/output/"

setwd(path2pred) # set working directory

####choose scale of your thematic map####
cellfactor=1 #factor'cellsize to aggregate cells to larger units, i.e. 5m raster and factor 10 = 50m raster. keep at one if no rescaling should be done

resolution=5*cellfactor

print(dir(pattern = ".tif"))

memory.limit(size= 200000) #incase you need to expand your memory


####import raster layers needed for input, change to fit your substrate compositions ####

boulders <- raster(dir(pattern = ".tif")[1])
clay <- raster(dir(pattern = ".tif")[2])
gravel <- raster(dir(pattern = ".tif")[3])
hard_clay <- raster(dir(pattern = ".tif")[4])
lboulders <- raster(dir(pattern = ".tif")[5])
large_stones <- raster(dir(pattern = ".tif")[6])
sand <- raster(dir(pattern = ".tif")[7])
silt <- raster(dir(pattern = ".tif")[8])
stones_pebbles <- raster(dir(pattern = ".tif")[9])


####enables rescaling of HUB and substrate input rasters####
dir.create(file.path(path2pred, paste(resolution,"m", sep='')), showWarnings = TRUE) #creates new folder for rescaled output of coverage rasters
dir.create(file.path(path2output, paste(resolution,"m", sep='')), showWarnings = TRUE) #creates new folder for rescaled output of HUB

boulders <- aggregate(boulders, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
clay <- aggregate(clay, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
gravel <- aggregate(gravel, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
hard_clay <- aggregate(hard_clay, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
lboulders <- aggregate(lboulders, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
large_stones <- aggregate(large_stones, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
sand <- aggregate(sand, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
silt <- aggregate(silt, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
stones_pebbles <- aggregate(stones_pebbles, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)

writeRaster(boulders, filename = paste(path2pred,resolution,"m/",names(boulders),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(clay, filename = paste(path2pred,resolution,"m/",names(clay),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(gravel, filename = paste(path2pred,resolution,"m/",names(gravel),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(hard_clay, filename = paste(path2pred,resolution,"m/",names(hard_clay),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(lboulders, filename = paste(path2pred,resolution,"m/",names(lboulders),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(large_stones, filename = paste(path2pred,resolution,"m/",names(large_stones),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(sand, filename = paste(path2pred,resolution,"m/",names(sand),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(silt, filename = paste(path2pred,resolution,"m/",names(silt),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(stones_pebbles, filename = paste(path2pred,resolution,"m/",names(stones_pebbles),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)


####end rescaling and writing to new resolution####

####stacking all input layers####

sediment_stack <- brick(boulders, clay, gravel, hard_clay, lboulders, large_stones, sand, silt, stones_pebbles)

# auxillary raster layers
 
  # "soft" sediment layers
    aux_soft_substrate <- sand + stones_pebbles + gravel + silt + clay
    aux_coarse_substrate <- stones_pebbles + gravel
    aux_ratio_muddy <- (silt + clay)/aux_soft_substrate
    aux_ratio_coarse <- aux_coarse_substrate / (sand + gravel + stone_pebbles)
    softsediment_stack <- stack(sand, aux_coarse_substrate, aux_soft_substrate, aux_greater_coarse)   
    
  # "hard" sediment layers
    aux_rockboulderstones <- boulders + large_stones +lboulders

#create rasterstack for all the sediment groups that will be used in Helcom Hub Level 3 classification
    
aux_sediment_stack <- brick(hard_clay, aux_rockboulderstones, aux_soft_substrate)

#to get the maximum value for each pixel in a sediment layer stacks stack
max_sed_value <-calc(aux_sediment_stack, function(x){max(x)})

#### Function to determine the code for Helcom Hub Level 3 ####

compare_function <- function(aux_soft_substrate, aux_coarse_substrate, aux_rockboulderstones, aux_ratio_coarse, aux_ratio_muddy, hard_clay, max_sed_value) { 
  ifelse(max_sed_value <= .8999999999, 5,       
         ifelse(aux_soft_substrate>= .9 & aux_ratio_muddy >=.2,6,
         ifelse(aux_soft_substrate >= .9 & aux_ratio_coarse >= .3, 3, 
                ifelse(aux_soft_substrate >= .9 & aux_ratio_coarse < .3, 4,
                       ifelse(aux_rockboulderstones >= .9 , 1,
                              ifelse(hard_clay >= .9, 2, 20))))))
  }
                                   

hub_level3 <- overlay(stack(aux_soft_substrate, aux_coarse_substrate, aux_rockboulderstones, aux_ratio_coarse, aux_ratio_muddy, hard_clay, max_sed_value), fun = compare_function) 

plot(hub_level3)


#### Create categorical data or raster attribute table (RAT) file ####
hub_level3 <- ratify(hub_level3)
hub_level3_rat <- levels(hub_level3)[[1]]
hub_level3_rat$Value <- c(1, 2, 3, 4, 5)
hub_level3_rat$Class_Name <- as.factor(c("rock and boulder", "hard clay", "coarse sediment", "sand", "mixed"))
hub_level3_rat$Class_Code <- as.factor(c("A", "B", "I", "J", "M"))
#hub_level3_rat$Count <- as.factor(c("400806",	"30739",	"8718585",	"19603528",	"25037088")) #example numbers of pixel count for each class, insert manually using ArcGIS calculated dbf file, or make a smarter script and pass along!:)
#hub_level3_rat$Class_km2 <- as.factor(c("10.02",	"0.76",	"217.96",	"490.08",	"625.92")) #example numbers of total area (km2) of each class,insert manually or make a smarter script if you want this...
#hub_level3_rat$Class_perc <- as.factor(c("0.745",	"0.056",	"16.208",	"36.444",	"46.545")) #example numbers or percent coverage of each class, insert manually or make a smarter script if you want this...
levels(hub_level3) <- hub_level3_rat
hub_level3_rat
write.dbf(hub_level3_rat, file = paste(path2output,resolution,"m/","hub_level3","_",resolution,"m.tif.vat.dbf", sep=''), factor2char = TRUE, max_nchar = 254)


#### write reclassified hub 3 map to file ####
writeRaster(hub_level3, filename = paste(path2output,resolution,"m/","hub_level3","_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT2S')

