
######### Supplemental material: reclassification from continuous models to HELCOM HUB LEVEL 2, 4, 5 and 6 thematic maps #########
#this script can also be modified to work on csv file to reclassify annotated percent coverage data# 
#run level 3 substrate reclassification seperatly before you run this at the same scale/pixel size
#enables rescaling to multiple resolutions
#creates labels for ArcGIS in the end of the script (dbf files)
#results in complete code level 1-6

####load packages####

library(raster)
library(sp)
library(rgdal)
library(foreign)


#### setting paths ####
path2pred = "path/compositional_models/"   #set path to biological component predictions to be reclassified
path2output = "path/output/"
version = "version_xx.tif" #change name

path2level3 = "path/hub3/"  #location of reclassified hub 3 map from step 6.
#name of your hub 3 raster at the scale of choice, make sure the same resolutions have already been run for hub 3
hub_level3 = raster(paste(path2level3,resolution,"m/","hub_level3","_",resolution,"m.tif", sep=''))


# set working directory
setwd(path2pred)


####multiscale foreloop, modify the list of scale factors and active if you want to do this step####
#for (i in c(1,2,5,10,50)) {

i=1 #scale factor, if left at 1 your pixel resolution will remain unchanged
cellfactor=i #factor'cellsize to aggregate cells to larger units, i.e. 5m raster and factor 10 = 50m raster. keep at one if no rescaling should be done
resolution=5*cellfactor


memory.limit(size= 200000)

####read input rasters####

#level2 input rasters 
depth = raster("path/depth.tif")
depth <- aggregate(depth, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)

#level 2 & 4-6 input rasters
print(dir(pattern = ".tif"))

unveg_substrate <- raster(dir(pattern = ".tif")[11]) #level 4

plot(unveg_substrate) # making sure correct input

annual_algae <- raster(dir(pattern = ".tif")[1]) #level 5 and 6 as S1
bivalves <- raster(dir(pattern = ".tif")[3]) #level 5 and 6
cnidarians <- raster(dir(pattern = ".tif")[4]) #level 5 and 6
moss_animals <- raster(dir(pattern = ".tif")[5]) #level 5 and 6
perennial_algae <- raster(dir(pattern = ".tif")[6]) #level 5
perennial_filamentous_algae <- raster(dir(pattern = ".tif")[7]) #level 6
perennial_foliose_red_algae <- raster(dir(pattern = ".tif")[8]) #level 6
perennial_nonfilamentous_corticated_red_algae <- raster(dir(pattern = ".tif")[9]) #level 6
soft_crustose_algae <- raster(dir(pattern = ".tif")[10]) #level 5

####enables scaling of HUB and downsampling of biological rasters####
dir.create(file.path(path2pred, paste(resolution,"m", sep='')), showWarnings = TRUE) #creates new folder for rescaled output of coverage rasters
dir.create(file.path(path2output, paste(resolution,"m", sep='')), showWarnings = TRUE) #creates new folder for rescaled output of HUB

unveg_substrate <- aggregate(unveg_substrate, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
annual_algae <- aggregate(annual_algae, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
bivalves <- aggregate(bivalves, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
cnidarians <- aggregate(cnidarians, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
moss_animals <- aggregate(moss_animals, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
perennial_algae <- aggregate(perennial_algae, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
perennial_filamentous_algae <- aggregate(perennial_filamentous_algae, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
perennial_foliose_red_algae <- aggregate(perennial_foliose_red_algae, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
perennial_nonfilamentous_corticated_red_algae <- aggregate(perennial_nonfilamentous_corticated_red_algae, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
soft_crustose_algae <- aggregate(soft_crustose_algae, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)

writeRaster(unveg_substrate, filename = paste(path2pred,resolution,"m/",names(unveg_substrate),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(annual_algae, filename = paste(path2pred,resolution,"m/",names(annual_algae),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(bivalves, filename = paste(path2pred,resolution,"m/",names(bivalves),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(cnidarians, filename = paste(path2pred,resolution,"m/",names(cnidarians),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(moss_animals, filename = paste(path2pred,resolution,"m/",names(moss_animals),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(perennial_algae, filename = paste(path2pred,resolution,"m/",names(perennial_algae),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(perennial_filamentous_algae, filename = paste(path2pred,resolution,"m/",names(perennial_filamentous_algae),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(perennial_foliose_red_algae, filename = paste(path2pred,resolution,"m/",names(perennial_foliose_red_algae),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(perennial_nonfilamentous_corticated_red_algae, filename = paste(path2pred,resolution,"m/",names(perennial_nonfilamentous_corticated_red_algae),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(soft_crustose_algae, filename = paste(path2pred,resolution,"m/",names(soft_crustose_algae),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)

####HUB 2 vertical zone ####
sum_algae <- perennial_algae + annual_algae + soft_crustose_algae

writeRaster(sum_algae, filename = paste(path2pred,resolution,"m/","sum_algae", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)#need to write the raster and read it back in otherwise it doesn't work in following functions
sum_algae <- raster(paste(path2pred,resolution,"m/","sum_algae", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''))

photic_depth = -43.389 #max depth where algae was predictedfound in our study, this is a tricky one but thats how we did...

#calculating hub level 2 based on the maximum depth of observed algae in groundtruth data, as well as predicted values of algae in the model
photic_zone_function <- function(sum_algae, depth) {
  ifelse(depth >= photic_depth, 1, 
         ifelse(sum_algae >= 0.01, 1, 2)) #this step ensure that if algae is predicted (>=0.01% = presence in our models) below the observed photic depth (photic_depth) it will be classified as a photic pixel
}

hub_level2 <- overlay(stack(sum_algae, depth), fun = photic_zone_function) 


#saves raster and reloads

writeRaster(hub_level2, filename = paste(path2output,resolution,"m/","hub_level2", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT4S')

hub_level2 <- raster(paste(path2output,resolution,"m/","hub_level2", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''))

####HUB 4 biological cover ####

compare_function_hub_level4 <- function(unveg_substrate) { 
  ifelse(unveg_substrate <= 0.9, 1,       
         ifelse(unveg_substrate > 0.9 & unveg_substrate <= 0.999, 2, 
                ifelse(unveg_substrate > 0.999, 4, 20)))
}

hub_level4 <- calc(unveg_substrate, fun=compare_function_hub_level4) #converts unveg_substrate raster to 1,2,4 values using function above


#saves raster and reloads

writeRaster(hub_level4, filename = paste(path2output,resolution,"m/","hub_level4", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT4S')

#writeRaster(hub_level4, filename = paste(path2output,resolution,"m/","hub_level4", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)#need to write the raster and read it back in otherwise it doesn't work in following functions

hub_level4 <- raster(paste(path2output,resolution,"m/","hub_level4", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''))

plot(hub_level4)

# Definitions Hub_level4 1 = 1, >= 10% coverage, 2 = 2, <10 > 0 % coverage (sparse), 4 = 4, 0% coverage


####HUB level 5 ####

aux_peren_stack <- brick(perennial_algae, bivalves, cnidarians, moss_animals) # makes stack/brick of level 5 perennial groups
max_peren_value <- calc(aux_peren_stack, function(x){max(x)}) # calculates a raster where cell values equal the maximum coverage value of the layers in peren_stack

writeRaster(max_peren_value, filename = paste(path2pred,resolution,"m/","max_peren_value","_",resolution,"m.tif", sep=''), overwrite=TRUE)

compare_function_hub_level5 <- function(hub_level4, max_peren_value, perennial_algae, bivalves, cnidarians, moss_animals, annual_algae, soft_crustose_algae) {
  ifelse(hub_level4 == 1 & max_peren_value >= 0.1 & perennial_algae >= bivalves & perennial_algae >= cnidarians & perennial_algae >= moss_animals, 1,
         ifelse(hub_level4 == 1 & max_peren_value >= 0.1 & bivalves >= perennial_algae & bivalves >= cnidarians & bivalves >= moss_animals, 2,
                ifelse(hub_level4 == 1 & max_peren_value >= 0.1 & cnidarians >= perennial_algae & cnidarians >= bivalves & cnidarians >= moss_animals, 3,
                       ifelse(hub_level4 == 1 & max_peren_value >= 0.1 & moss_animals >= perennial_algae & moss_animals >= bivalves & moss_animals >= cnidarians, 4,
                              ifelse(hub_level4 == 1 & max_peren_value < 0.1 & soft_crustose_algae >= 0.1, 5,
                                     ifelse(hub_level4 == 1 & max_peren_value < 0.1 & soft_crustose_algae < 0.1 & annual_algae >= 0.1, 6,
                                            ifelse(hub_level4 == 1 & max_peren_value < 0.1 & soft_crustose_algae < 0.1 & annual_algae < 0.1 & (annual_algae + bivalves + cnidarians + moss_animals + perennial_algae + soft_crustose_algae) >= 0.05, 7,
                                                   ifelse(hub_level4 == 1, 15, 
                                                          ifelse(hub_level4 == 2, 8,
                                                                 ifelse(hub_level4 == 4, 9, 20))))))))))
}


# If unvegetated substrate and biological components does not match up you will get 15 or 20 (NA), to fix this you have to fix the source rasters so that sum of biological comoents are not smaller then 1- unvegetated (i.e. uncolonized)

#Overlay function to produce a single raster layer using compare function above 
hub_level5 <- overlay(stack(hub_level4, max_peren_value, perennial_algae, bivalves, cnidarians, moss_animals, annual_algae, soft_crustose_algae), fun = compare_function_hub_level5) 


#saves raster and reloads

writeRaster(hub_level5, filename = paste(path2output,resolution,"m/","hub_level5", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT4S')

hub_level5 <- raster(paste(path2output,resolution,"m/","hub_level5", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''))

plot(hub_level5)

#### HUB level 6 ####

#Compare function to group raster cells based on level 6 split rules 
compare_function_hub_level6 <- function(hub_level5, perennial_filamentous_algae, perennial_foliose_red_algae, perennial_nonfilamentous_corticated_red_algae, bivalves, cnidarians, moss_animals, annual_algae, soft_crustose_algae) { 
  ifelse(hub_level5 == 1 & perennial_filamentous_algae >= 0.5 & perennial_filamentous_algae >= perennial_foliose_red_algae & perennial_filamentous_algae >= perennial_nonfilamentous_corticated_red_algae, 1,       
         ifelse(hub_level5 == 1 & perennial_foliose_red_algae >= 0.5 & perennial_foliose_red_algae >= perennial_filamentous_algae & perennial_foliose_red_algae >= perennial_nonfilamentous_corticated_red_algae, 2, 
                ifelse(hub_level5 == 1 & perennial_nonfilamentous_corticated_red_algae >= 0.5 & perennial_nonfilamentous_corticated_red_algae >= perennial_filamentous_algae & perennial_nonfilamentous_corticated_red_algae >= perennial_foliose_red_algae, 3,
                       ifelse(hub_level5 == 2 & bivalves >= 0.5, 4,
                              ifelse(hub_level5 == 3 & cnidarians >= 0.5, 5,
                                     ifelse(hub_level5 == 4 & moss_animals >= 0.5, 6,
                                            ifelse(hub_level5 == 5 & soft_crustose_algae >= 0.5, 7,
                                                   ifelse(hub_level5 == 6 & annual_algae >= 0.5, 8,
                                                          ifelse(hub_level5 == 7, 0,
                                                                 ifelse(hub_level5 == 8, 0,
                                                                        ifelse(hub_level5 == 9, 0, 0)))))))))))
}

#Note: hub_level6 definitions are C5=1, C3=2, C2=3, E1=4, G1=5, H1=6, R=7, S1=8,  9-13, all these levels already in level 5, set to 0 in code for level 6 : Previous code (V=9, T=10, U=11, 13=NA there can and should be 13)
#Note: have used Annual_algae for S1 as data shows that Chorda filum and Haliosiphon (S2) only occur at 12 sites and their coverage make up max 4% of the annual algae coverage i.e. the rest is filamentous.
#This can be used to multiply rasters: hub_level4, hub_level5 and hub_level6 to form a raster with unique values to classes

hub_level6 <- overlay(stack(hub_level5, perennial_filamentous_algae, perennial_foliose_red_algae, perennial_nonfilamentous_corticated_red_algae, bivalves, cnidarians, moss_animals, annual_algae, soft_crustose_algae), fun = compare_function_hub_level6) 
writeRaster(hub_level6, filename = paste(path2output,resolution,"m/","hub_level6", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT4S')
hub_level6 <- raster(paste(path2output,resolution,"m/","hub_level6", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''))

plot(hub_level6)

#### HUB level 4-5 ####
hub_level45 <- (hub_level4 * 10000) + (hub_level5 * 100)
writeRaster(hub_level45, filename = paste(path2output,resolution,"m/","hub_level4-5", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT4S')
hub_level45<- raster(paste(path2output,resolution,"m/","hub_level4-5", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''))


plot(hub_level45)
#table(as.data.frame(hub_level45))

#### HUB level 4-6v####
hub_level46 <- hub_level45 + hub_level6 
writeRaster(hub_level46, filename = paste(path2output,resolution,"m/","hub_level4-6", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT4S')
hub_level46 <- raster(paste(path2output,resolution,"m/","hub_level4-6", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''))

plot(hub_level46)

####HUB level 1-6 #### 
#you have to run and read HUB level 3 in the corresponding resolution before this works

#level 1 "baltic" is given value 1000000
#All HUB levels concatenated 
hub_level16 <- (100000000 + (hub_level2 * 10000000) + (hub_level3 * 100000) + hub_level46)
writeRaster(hub_level16, filename = paste(path2output,resolution,"m/","hub_level1-6", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT4S')
hub_level16 <- raster(paste(path2output,resolution,"m/","hub_level1-6", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''))

plot(hub_level16)

####end multiresolution forloop#### 
#if you want to run this for multiple resolutions
#} 

####CREATE LABELS for ArcGIS####
#read in rasters if you have not run the script above in the same session
#resolution=5
#hub_level2 <- raster(paste(path2output,resolution,"m/","hub_level2", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''))
#hub_level3 <- raster(paste(path2output,resolution,"m/","hub_level3","_",resolution,"m.tif", sep=''))
#hub_level4 <- raster(paste(path2output,resolution,"m/","hub_level4", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''))
#hub_level5 <- raster(paste(path2output,resolution,"m/","hub_level5", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''))
#hub_level6 <- raster(paste(path2output,resolution,"m/","hub_level6", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''))
#hub_level16 <- raster(paste(path2output,resolution,"m/","hub_level1-6", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''))


###create arcgis dbf labels for level 2###
hub_level2 <- ratify(hub_level2)
hub_level2_rat <- levels(hub_level2)[[1]]
hub_level2_rat$Value <- c(1, 2)
hub_level2_rat$Class_Name <- as.factor(c("photic benthos", "aphotic benthos"))
hub_level2_rat$Class_Code <- as.factor(c("A", "B"))
levels(hub_level2) <- hub_level2_rat

write.dbf(hub_level2_rat, file = paste(path2output,resolution,"m/","hub_level2", strsplit(version,".tif"),"_",resolution,"m.tif.vat.dbf", sep=''), factor2char = TRUE, max_nchar = 254)
#writeRaster(hub_level2, filename = paste(path2output,resolution,"m/","hub_level2", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT4S')
hub_level2_rat

###create arcgis dbf labels for level 4###
hub_level4 <- ratify(hub_level4)
hub_level4_rat <- levels(hub_level4)[[1]]
hub_level4_rat$Value <- c(1, 2, 4)
hub_level4_rat$Class_Name <- as.factor(c("macroscopic epibenthic biotic structures (>= 10% coverage)", "sparse macroscopic epibenthic biotic structures (<10% coverage)", "no macroscopic epibenthic biotic structures (0% cover)" ))
hub_level4_rat$Class_Code <- as.factor(c("1", "2", "4"))
levels(hub_level4) <- hub_level4_rat
write.dbf(hub_level4_rat, file = paste(path2output,resolution,"m/","hub_level4", strsplit(version,".tif"),"_",resolution,"m.tif.vat.dbf", sep=''), factor2char = TRUE, max_nchar = 254)
hub_level4_rat
#writeRaster(hub_level4, filename = paste(path2output,resolution,"m/","hub_level4", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT4S')


###create arcgis dbf labels for level 5###
hub_level5 <- ratify(hub_level5)
hub_level5_rat <- levels(hub_level5)[[1]]
hub_level5_rat$Value <- c(1, 2, 3, 6, 7, 8, 9)
hub_level5_rat$Class_Name <- as.factor(c("perennial algae", "epibenthic bivalves", "epibenthic cnidarians","annual algae", "mixed epibenthic community","sparse epibenthic community", "no epibenthic community"))
hub_level5_rat$Class_Code <- as.factor(c("C", "E", "G", "S", "V", "T", "U"))
levels(hub_level5) <- hub_level5_rat
write.dbf(hub_level5_rat, file = paste(path2output,resolution,"m/","hub_level5", strsplit(version,".tif"),"_",resolution,"m.tif.vat.dbf", sep=''), factor2char = TRUE, max_nchar = 254)
hub_level5_rat
#writeRaster(hub_level5, filename = paste(path2output,resolution,"m/","hub_level5", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT4S')


###create arcgis dbf labels for level 6###
hub_level6 <- ratify(hub_level6)
hub_level6_rat <- levels(hub_level6)[[1]]
hub_level6_rat$Value <- c(0, 1, 4, 5, 8)
hub_level6_rat$Class_Name <- as.factor(c("NA", "perennial filamentous algae", "mytilidae", "cnidarians", "annual algae"))
hub_level6_rat$Class_Code <- as.factor(c("", "C5", "E1", "G1", "S1"))
levels(hub_level6) <- hub_level6_rat
write.dbf(hub_level6_rat, file = paste(path2output,resolution,"m/","hub_level6", strsplit(version,"m.tif"),"_",resolution,".tif.vat.dbf", sep=''), factor2char = TRUE, max_nchar = 254)
#writeRaster(hub_level6, filename = paste(path2output,resolution,"m/","hub_level6", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT4S')


###create arcgis dbf labels for level 45###
hub_level45 <- ratify(hub_level45)
hub_level45_rat <- levels(hub_level45)[[1]]
hub_level45_rat
hub_level45_rat$Value <- c(10100, 10200, 10300, 10600, 10700, 20800, 40900)
hub_level45_rat$Class_Name <- as.factor(c("characterised by perennial algae", "characterised by epibenthic bivalves", "characterised by epibenthic cnidarians", "characterised by annual algae", "mixed epibenthic community", "sparse epibenthic community", "no epibenthic community"))
hub_level45_rat$Class_Code <- as.factor(c("1C", "1E", "1G", "1S", "V", "T", "U"))
levels(hub_level45) <- hub_level45_rat
write.dbf(hub_level45_rat, file = paste(path2output,resolution,"m/","hub_level4-5", strsplit(version,".tif"),"_",resolution,"m.tif.vat.dbf", sep=''), factor2char = TRUE, max_nchar = 254)
#writeRaster(hub_level45, filename = paste(path2output,resolution,"m/","hub_level45", strsplit(version,".tif"),"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT4S')

###create arcgis dbf labels for level 46###
hub_level46 <- ratify(hub_level46)
hub_level46_rat <- levels(hub_level46)[[1]]
hub_level46_rat
hub_level46_rat$Value <- c(10100,	10101,	10200,	10204,	10300,	10305,	10600,	10608,	10700,	20800, 40900)
hub_level46_rat$Class_Name <- as.factor(c("characterised by perennial algae",	"dominated by perennial filamentous algae",	"characterised by epibenthic bivalves",	"dominated by mytilidae", "characterised by cniderians", "dominated by cniderians", "characterised by annual algae",	"dominated by annual algae",	"mixed epibenthic community",	"sparse epibenthic community",	"no epibenthic community"))
hub_level46_rat$Class_Code <- as.factor(c("1C",	"1C5",	"1E",	"1E1",	"1G",	"1G1",	"1S",	"1S1",	"1V",	"2T",	"4U"))
levels(hub_level46) <- hub_level46_rat
write.dbf(hub_level46_rat, file = paste(path2output,resolution,"m/","hub_level4-6", strsplit(version,".tif"),"_",resolution,"m.tif.vat.dbf", sep=''), factor2char = TRUE, max_nchar = 254)
#writeRaster(hub_level45, filename = paste(path2output,resolution,"m/","hub_level6", strsplit(version,"m.tif"),"_",resolution,".tif.vat.dbf", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT4S')

###create arcgis dbf labels for level 16###
hub_level16 <- ratify(hub_level16)
hub_level16_rat <- levels(hub_level16)[[1]]
hub_level16_rat
hub_level16_rat$Value <- c(110110100, 	110110101, 	110110200, 	110110204, 	110110300, 	110110305, 	110110700, 	110120800, 	110210100, 	110210101, 	110210200, 	110210204, 	110210300, 	110210700, 	110220800, 	110240900, 	110310100, 	110310101, 	110310200, 	110310204, 	110310300, 	110310600, 	110310700, 	110320800, 	110340900, 	110410100, 	110410200, 	110410300, 	110410600, 	110410700, 	110420800, 	110440900, 	110510100, 	110510101, 	110510200, 	110510204, 	110510300, 	110510305, 	110510600, 	110510608, 	110510700, 	110520800, 	110540900, 	120110200, 	120110300, 	120110700, 	120210700, 	120220800, 	120240900, 	120320800, 	120340900, 	120420800, 	120440900, 	120510200, 	120510300, 	120510700, 	120520800)
hub_level16_rat$Class_Name <- as.factor(c("Baltic photic rock and boulder characterised by perennial algae", 	"Baltic photic rock and boulder dominated by perennial filamentous algae", 	"Baltic photic rock and boulder characterised by epibenthic bivalves", 	"Baltic photic rock and boulder dominated by mytilidae", 	"Baltic photic rock and boulder characterized by cniderians", 	"Baltic photic rock and boulder dominated by cniderians", 	"Baltic photic rock and boulder mixed epibenthic community", 	"Baltic photic rock and boulder sparse epibenthic community", 	"Baltic photic hard clay characterised by perennial algae", 	"Baltic photic hard clay dominated by perennial filamentous algae", 	"Baltic photic hard clay characterised by epibenthic bivalves", 	"Baltic photic hard clay dominated by mytilidae", 	"Baltic photic hard clay characterized by cniderians", 	"Baltic photic hard clay mixed epibenthic community", 	"Baltic photic hard clay sparse epibenthic community", 	"Baltic photic hard clay no epibenthic community", 	"Baltic photic coarse substrate characterised by perennial algae", 	"Baltic photic coarse substrate dominated by perennial filamentous algae", 	"Baltic photic coarse substrate characterised by epibenthic bivalves", 	"Baltic photic coarse substrate dominated by mytilidae", 	"Baltic photic coarse substrate characterized by cniderians", 	"Baltic photic coarse substrate dominated by cniderians", 	"Baltic photic coarse substrate mixed epibenthic community", 	"Baltic photic coarse substrate sparse epibenthic community", 	"Baltic photic coarse substrate no epibenthic community", 	"Baltic photic sand characterised by perennial algae", 	"Baltic photic sand characterised by epibenthic bivalves", 	"Baltic photic sand characterized by cniderians", 	"Baltic photic sand dominated by cniderians", 	"Baltic photic sand mixed epibenthic community", 	"Baltic photic sand sparse epibenthic community", 	"Baltic photic sand no epibenthic community", 	"Baltic photic mixed characterised by perennial algae", 	"Baltic photic mixed dominated by perennial filamentous algae", 	"Baltic photic mixed characterised by epibenthic bivalves", 	"Baltic photic mixed dominated by mytilidae", 	"Baltic photic mixed characterized by cniderians", 	"Baltic photic mixed dominated by cniderians", 	"Baltic photic mixed dominated by cniderians", 	"Baltic photic mixed dominated by annual algae", 	"Baltic photic mixed mixed epibenthic community", 	"Baltic photic mixed sparse epibenthic community", 	"Baltic photic mixed no epibenthic community", 	"Baltic aphotic rock and boulder characterised by epibenthic bivalves", 	"Baltic aphotic rock and boulder characterized by cniderians", 	"Baltic aphotic rock and boulder mixed epibenthic community", 	"Baltic aphotic hard clay mixed epibenthic community", 	"Baltic aphotic hard clay sparse epibenthic community", 	"Baltic aphotic hard clay no epibenthic community", 	"Baltic aphotic coarse substrate sparse epibenthic community", 	"Baltic aphotic coarse substrate no epibenthic community", 	"Baltic aphotic sand sparse epibenthic community", 	"Baltic aphotic sand no epibenthic community", 	"Baltic aphotic mixed characterised by epibenthic bivalves", 	"Baltic aphotic mixed characterized by cniderians", 	"Baltic aphotic mixed mixed epibenthic community", 	"Baltic aphotic mixed sparse epibenthic community"))
hub_level16_rat$Class_Code <- as.factor(c("AA.A1C", 	"AA.A1C5", 	"AA.A1E", 	"AA.A1E1", 	"AA.A1G", 	"AA.A1G1", 	"AA.A1V", 	"AA.A2T", 	"AA.B1C", 	"AA.B1C5", 	"AA.B1E", 	"AA.B1E1", 	"AA.B1G", 	"AA.B1V", 	"AA.B2T", 	"AA.B4U", 	"AA.I1C", 	"AA.I1C5", 	"AA.I1E", 	"AA.I1E1", 	"AA.I1G", 	"AA.I1G1", 	"AA.I1V", 	"AA.I2T", 	"AA.I4U", 	"AA.J1C", 	"AA.J1E", 	"AA.J1G", 	"AA.J1G1", 	"AA.J1V", 	"AA.J2T", 	"AA.J4U", 	"AA.M1C", 	"AA.M1C5", 	"AA.M1E", 	"AA.M1E1", 	"AA.M1G", 	"AA.M1G1", 	"AA.M1G1", 	"AA.M1S1", 	"AA.M1V", 	"AA.M2T", 	"AA.M4U", 	"AB.A1E", 	"AB.A1G", 	"AB.A1V", 	"AB.B1V", 	"AB.B2T", 	"AB.B4U", 	"AB.I2T", 	"AB.I4U", 	"AB.J2T", 	"AB.J4U", 	"AB.M1E", 	"AB.M1G", 	"AB.M1V", 	"AB.M2T"))
levels(hub_level16) <- hub_level16_rat
write.dbf(hub_level16_rat, file = paste(path2output,resolution,"m/","hub_level1-6", strsplit(version,".tif"),"_",resolution,"m.tif.vat.dbf", sep=''), factor2char = TRUE, max_nchar = 254)
#writeRaster(hub_level45, filename = paste(path2output,resolution,"m/","hub_level6", strsplit(version,"m.tif"),"_",resolution,".tif.vat.dbf", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT4S')

