
######### Supplemental material: reclassification from continuous models to Natura 2000 thematic maps (sandbank 1110 & reefs 1170) #########
#this script can also be modified to work on csv file to reclassify annotated percent coverage data# 
#includes an experimental algoritm to classify Natura 2000 subtypes, including some additional features of interest found on Hoburgs Bank
#enables rescaling to multiple resolutions
#creates labels for ArcGIS in the end of the script (dbf files)


library(raster)
library(sp)
library(rgdal)
library(foreign)

#Activate forloop to produce multiple resolution maps 
#for (i in c(5,10,50)) {
i=1
#set resolution
cellfactor=i #factor'cellsize to aggregate cells to larger units, i.e. 5m raster and factor 10 = 50m raster. keep at one if no rescaling should be done
resolution=5*cellfactor

#### setting paths ####
way = "path/"
folder = "foldername"
path2pred = paste(path, "substrate/", folder,"/", resolution, "m/", sep='')
path2bivalves = paste(way, "biota/", folder,"/", resolution, "m/", sep='')
path2output = paste(way, "natura2000/",resolution,"m/", sep="")
path2outputdir = paste(way,"natura2000/", sep="")

## set working directory
setwd(path2pred)

memory.limit(size= 2000000)


####create new directory for multiscale natura 2000 rasters and aggregated substrate layers####
dir.create(file.path(path2pred, paste("aggregated", sep='')), showWarnings = TRUE) #creates new folder for rescaled output of coverage rasters
dir.create(file.path(path2outputdir, paste(resolution,"m", sep='')), showWarnings = TRUE) #creates new folder for rescaled output of HUB

####import raster layers needed for input ####
print(dir(pattern = ".tif"))

boulders <- raster(dir(pattern = ".tif")[1])
clay <- raster(dir(pattern = ".tif")[2])
gravel <- raster(dir(pattern = ".tif")[3])
hard_clay <- raster(dir(pattern = ".tif")[4])
lboulders <- raster(dir(pattern = ".tif")[5])
large_stones <- raster(dir(pattern = ".tif")[6])
sand <- raster(dir(pattern = ".tif")[7])
silt <- raster(dir(pattern = ".tif")[8])
stones_pebbles <- raster(dir(pattern = ".tif")[9])

bivalves <- raster(paste(path2bivalves,"epibenthic_bivalves_"),resolution,"m.tif", sep="")
#### combine sediment layers####

hardbottom <- large_stones + boulders + lboulders + hard_clay
sand_gravel <- sand + gravel + stones_pebbles
softbottom <- silt + clay

writeRaster(hardbottom, filename = paste(path2pred,"aggregated/","hardbottom", folder,"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(sand_gravel, filename = paste(path2pred,"aggregated/","sand_gravel", folder,"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)
writeRaster(softbottom, filename = paste(path2pred,"aggregated/","softbottom", folder,"_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW)

hardbottom <- raster(paste(path2pred,"aggregated/","hardbottom", folder,"_",resolution,"m.tif", sep=''))
sand_gravel <- raster(paste(path2pred,"aggregated/","sand_gravel", folder,"_",resolution,"m.tif", sep=''))
softbottom <- raster(paste(path2pred,"aggregated/","softbottom", folder,"_",resolution,"m.tif", sep=''))

####Function to determine the natura 2000 type on hoburgs bank" ####

natura2000_function <- function(hardbottom, sand_gravel, softbottom, bivalves) { 
  ifelse(hardbottom >= 0.5 & bivalves<=1, 1,       
         ifelse(bivalves >= 0.1 & hardbottom < 0.5 ,1,
              ifelse(sand_gravel >= softbottom & bivalves<=0.1, 2, 3 )))
  }
                                   

natura_2000 <- overlay(stack(hardbottom, sand_gravel, softbottom, bivalves), fun = natura2000_function) 

plot(natura_2000)

writeRaster(natura_2000, filename = paste(path2output,"natura_2000","_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT2S')


#### subtypes, this is an experimental script, and represent our best attempt to mimic and ad to the definitions for Natura 2000 subtypes####
#read additional inputs"
#identify reef ridges based on bpi with inner radius 2.5m, outer radious 25m (generate in seperate step outside this script)
path2reef <- "path/metrics/"
dist_reef <- raster(paste(path2reef,"reefbpi_eucldist.tif", sep=""))
dist_reef <- aggregate(dist_reef, fact=cellfactor, fun=mean, expand=TRUE, na.rm=TRUE)
distreef <-dist_reef
plot(distreef)

#identify moving and accumulating sand areas based on multinomial sandripple model if you have this available...
path2sandclass <- "path/metrics/"
sandclass <- raster(paste(path2sandclass,"sandripples.tif", sep=""))
sandclass <- aggregate(sandclass, fact=cellfactor, fun=median, expand=TRUE, na.rm=TRUE)
plot(sandclass)

#reading hub level 2 if to include photic zone in natura 2000 definition, disabled for now
#path2hub2 = paste("path/hub/", folder,"/",typeofmodel, "/", resolution, "m/", sep='')
#hub2 <- raster(paste(path2hub2,"hub_level2_", typeofmodel, "_",folder,"_",resolution, "m.tif", sep=""))
#plot(hub2)



natura2000subtype_function <- function(natura_2000, boulders, lboulders, hard_clay, sandclass, dist_reef, bivalves) { 
  ifelse(natura_2000 == 1 & dist_reef >= -100 & hard_clay >=0.5 & boulders >= 0 & lboulders >= 0 & sandclass >= 0 & bivalves >= 0.1, 41,  
         ifelse(natura_2000 == 1 & dist_reef >= -100 & hard_clay >=0.5 & boulders >= 0 & lboulders >= 0 & sandclass >= 0 & bivalves < 0.1, 40,  
            ifelse(natura_2000 == 1 & dist_reef <=5 & hard_clay <0.5 & boulders >=0 & lboulders >=0 & sandclass >=0 & bivalves >= 0.1, 31,
                ifelse(natura_2000 == 1 & dist_reef <=5 & hard_clay <0.5 & boulders >=0 & lboulders >=0 & sandclass >=0 & bivalves < 0.1, 30,
                    ifelse(natura_2000 == 1 & dist_reef >=-100 & hard_clay >=0 & ((boulders + lboulders)>=0.02 | lboulders >= 0.002) & sandclass >= 0 & bivalves >= 0.1, 21,
                       ifelse(natura_2000 == 1 & dist_reef >=-100 & hard_clay >=0 & ((boulders + lboulders)>=0.02 | lboulders >= 0.002) & sandclass >= 0 & bivalves < 0.1, 20,
                          ifelse(natura_2000 == 1 & dist_reef >= -100 & hard_clay >= 0 & boulders >= 0 & lboulders >= 0 & sandclass >= 0 & bivalves >= 0.1, 10,
                              ifelse(natura_2000 == 1 & dist_reef >= -100 & hard_clay >= 0 & boulders >= 0 & lboulders >= 0 & sandclass >= 0 & bivalves < 0.1, 11,
                                ifelse(natura_2000 == 2 & (sandclass > 1 | sandclass == 0) & bivalves >= 0.002, 51,
                                     ifelse(natura_2000 == 2 & (sandclass > 1 | sandclass == 0) & bivalves < 0.002, 50,
                                            ifelse(natura_2000 == 2 & sandclass <= 1 & bivalves >= 0.002, 61,
                                                   ifelse(natura_2000 == 2 & sandclass <= 1 & bivalves < 0.002, 60,
                                                          ifelse(natura_2000 == 3 & dist_reef >= -100 & hard_clay >= 0 & boulders >= 0 & lboulders >= 0 & sandclass < 00, 00, 99)))))))))))))
}


natura_2000_subtype <- overlay(stack(natura_2000, boulders, lboulders, hard_clay, sandclass, dist_reef, bivalves), fun = natura2000subtype_function) 

geo_origin=100  #Geogenic origin = 100, biogenic = 200, in our case the whole area consisted of reefs of geogenic origin...

natura_2000_subtype <- natura_2000_subtype + geo_origin

plot(natura_2000_subtype)

writeRaster(natura_2000_subtype, filename = paste(path2output,"natura_2000_subtype","_",resolution,"m.tif", sep=''), overwrite=TRUE, COMPRESS=LZW, datatype = 'INT2S')

natura_2000_subtype <- raster(paste(path2output,"natura_2000_subtype","_",resolution,"m.tif", sep=''))

# natura2000_subtype 1= flat, 2=rock and boulder, 3 =ridge 4= clayreef, 5 = sand w ripples, 6 = sand no ripples, 10 = soft clay, 20 = NA



#### Create categorical data or raster attribute table (RAT) file for natura 2000 types original resolution####

natura_2000 <- ratify(natura_2000)
natura_2000_rat <- levels(natura_2000)[[1]]
##natura_2000_rat$Value <- c(1, 2, 3) #activate only when all 3 classes available
natura_2000_rat$Value <- c(1, 2)
#natura_2000_rat$Class_Name <- as.factor(c("reef", "sandbank", "soft sediments")) #activate only when all 3 classes available
#natura_2000_rat$Class_Code <- as.factor(c("1170", "1110", "1140")) #activate only when all 3 classes available
natura_2000_rat$Class_Name <- as.factor(c("reef", "sandbank"))
natura_2000_rat$Class_Code <- as.factor(c("1170", "1110"))
levels(natura_2000) <- natura_2000_rat
write.dbf(natura_2000_rat, file = paste(path2output,"natura_2000","_",resolution,"m.tif.vat.dbf", sep=''), factor2char = TRUE, max_nchar = 254)

#### Create categorical table (RAT) file for natura_2000_subtypes ####
natura_2000_subtype <- ratify(natura_2000_subtype)
natura_2000_subtype_rat <- levels(natura_2000_subtype)[[1]]
natura_2000_subtype_rat$Value <- c(110, 111, 120, 121, 130, 131, 140, 141, 150, 151, 160, 161)
natura_2000_subtype_rat$Class_Name <- as.factor(c("flat reef, geogenic", "flat reef w mussels, geogenic", "rock & boulder reef, geogenic", "rock & boulder reef w mussels, geogenic", "ridge reef, geogenic", "ridge reef w mussels, geogenic", "clay reef, geogenic", "clay reef w mussels, geogenic", "sandbank, rippels", "sandbank w mussels, ripples", "sandbank, no ripples", "sandbank w mussels, no ripples"))
natura_2000_subtype_rat$Class_Code <- as.factor(c("1170", "1170","1170","1170","1170", "1170","1170","1170", "1110", "1110", "1110", "1110"))
levels(natura_2000_subtype) <- natura_2000_subtype_rat


write.dbf(natura_2000_subtype_rat, file = paste(path2output,"natura_2000_subtype","_",resolution,"m.tif.vat.dbf", sep=''), factor2char = TRUE, max_nchar = 254)

#}


