######### Supplemental material: Accuracy of continous maps #########

####load library####
library(maptools)
library (raster)
library(xlsx)
library(gdata)
library(gplots)
library(ROCR)
library(sp)
library(dplyr)
library(ggplot2)
library(car)
library(DescTools)


#### setting paths####
setwd("path/aa/") #set working directory
path2aapoints = "path/points/"
path2rasteroutputs = "path/substrate/"
path2biorasteroutputs = "path/biology/"
path2aaoutputs = "path/aa/"
testdata = "testdata.csv"
path2testdata = paste(path2aapoints, testdata, sep='')
setwd(path2rasteroutputs)
print(dir(pattern = ".tif$"))


####read substrate components####
boulders <- raster(dir(pattern = ".tif$")[1]) #level 5 and 6 as S1
#clay <- raster(dir(pattern = ".tif$")[2]) #level 5 and 6
gravel <- raster(dir(pattern = ".tif$")[3]) #level 5 and 6
hard_clay <- raster(dir(pattern = ".tif$")[4])
large_boulders <- raster(dir(pattern = ".tif$")[5])
large_stones <- raster(dir(pattern = ".tif$")[6])
sand <- raster(dir(pattern = ".tif$")[7])
sand_silt_clay_comb <- raster(dir(pattern = ".tif$")[8]) #(sand + silt + clay) is the sand layer predicted from point intersept dropcamera data, which also included finer subfractions not possiblt to seperate in imagery
#silt <- raster(dir(pattern = ".tif$")[9])
stones_pebbles <- raster(dir(pattern = ".tif$")[10])

####read biological components####
setwd(path2biorasteroutputs)
print(dir(pattern = ".tif$")) # prints files in the directory that end in .tif

annual_algae <- raster(dir(patter = ".tif$")[1])
#detritus <- raster(dir(pattern = ".tif$")[2])
epibenthic_bivalves <- raster(dir(pattern = ".tif$")[2])
epibenthic_cnidarians <- raster(dir(pattern = ".tif$")[3])
epibenthic_moss_animals <- raster(dir(pattern = ".tif$")[4])
#epibenthic_sponges <- raster(dir(pattern = ".tif$")[6])  
perennial_algae <- raster(dir(pattern = ".tif$")[6])
perennial_filamentous_algae <- raster(dir(pattern = ".tif$")[7])
perennial_foliose_red_algae <- raster(dir(pattern = ".tif$")[8])
perennial_nonfilamentous_corticated_red_algae <- raster(dir(pattern = ".tif$")[9])
soft_crustose_algae <- raster(dir(pattern = ".tif$")[10])
uncolonized_substrate <- raster(dir(pattern = ".tif$")[12])



#### run accuracy for each component manually#### 
#set up a forloop if needed, this version is not so elegant...#

####CHANGE to your component of choice!####
rasterpred = "name_of_component_raster_to_be_tested.tif" #CHANGE
#path2rasterpred = paste(path2biorasteroutputs, rasterpred, sep='') #use for biology
path2rasterpred = paste(path2rasteroutputs, rasterpred, sep='') #use for substrate

path2aatable = paste(path2aaoutputs, "AAsumtable_", strsplit(rasterpred, ".tif"), ".csv", sep='')
path2aaplot = paste(path2aaoutputs, "AAplots_", strsplit(rasterpred, ".tif"), ".jpg", sep='')
path2aastattable = paste(path2aaoutputs, "AAstattable_", strsplit(rasterpred, ".tif"), ".csv", sep='') 

#### Import AA pts, intersect with and extract values from habitat map ####
#import AA pts
aapts <- read.csv(paste(path2testdata, sep=''), header=T, sep=",")
aapts <- as.data.frame(aapts)

#import habitat map prediction	
habmap <- raster(path2rasterpred)

#change columns names if needed, for example:
names(aapts)
colnames(aapts)[14]<-"hard_clay"
colnames(aapts)[17]<-"sand_silt_clay_comb"
colnames(aapts)[18]<-"gravel"
colnames(aapts)[19]<-"stones_pebbles"
colnames(aapts)[20]<-"large_stones"
colnames(aapts)[21]<-"boulders"
colnames(aapts)[22]<-"large_boulders"
colnames(aapts)[25]<-"uncolonized_substrate"
colnames(aapts)[26]<-"detritus"
colnames(aapts)[57]<-"annual_algae"
colnames(aapts)[58]<-"epibenthic_bivalves"
colnames(aapts)[59]<-"epibenthic_cnidarians"
colnames(aapts)[60]<-"epibenthic_moss_animals"
colnames(aapts)[61]<-"epibenthic_sponges"
colnames(aapts)[63]<-"perennial_algae"
colnames(aapts)[64]<-"soft_crustose_algae"
colnames(aapts)[65]<-"perennial_filamentous_algae"
colnames(aapts)[66]<-"perennial_foliose_red_algae"
colnames(aapts)[67]<-"perennial_nonfilamentous_corticated_red_algae"



####separate table into different habitat classes####

####CHANGE to your aa class of choice!####
aaclass <- subset(aapts, select=c(X_SWE99TM, Y_SWE99TM, aapts_class_name)) #change name to your aapts class (i.e. "epibenthic bivalves")

coordinates(aaclass) = ~ X_SWE99TM + Y_SWE99TM #convert test data into spatial object

#extract values from predicted habitat map at AA locations
o <- extract(habmap, aaclass, method="simple", sp=TRUE)
#view prediction and AA point locations
#plot(habmap, col=topo.colors(100))#plot prediction
#points(o) #overlay AA points
#title(main = "sand_silt_clay_comb", font.main = 4)#add title !!!CHANGE!!!

#format table
aaclass <- as.data.frame(o) #convert to data frame

aaclass <- na.omit(aaclass) #remove NA values
#row.names(aaclass) <- NULL #remove row names if exist

colnames(aaclass)[3]<-"CLASS_AA" #change column name
colnames(aaclass)[4]<-"CLASS_MAP" #change column name

aaclass[4] <- aaclass[4]*100 # changes CLASS_MAP from proportions to percent

obsValues_test <- aaclass$CLASS_AA
predValues_test <- aaclass$CLASS_MAP

#### statistics ####
#R-squared 
R2<-caret::R2(aaclass$CLASS_AA, aaclass$CLASS_MAP)
R2
#RMSE root-mean-square error 
RMSE<-caret::RMSE(aaclass$CLASS_AA, aaclass$CLASS_MAP)
RMSE

#MAE mean absolute error
MAE<-caret::MAE(aaclass$CLASS_AA, aaclass$CLASS_MAP)
MAE
#BIAS
BIAS <- mean(aaclass$CLASS_AA-aaclass$CLASS_MAP)
BIAS

summary_stats <- data.frame(R2,RMSE,MAE, BIAS) #converts to table
summary_stats
write.table(summary_stats, file = (paste(path2aastattable,sep="")), row.names=FALSE, col.names=TRUE, sep=",") #saves table as .csv


#### Error summary statistics based on class intervals ####
plot(aaclass$CLASS_AA~aaclass$CLASS_MAP, xlim = c(0,100), ylim = c(0,100))
aaclass$abserrp_o <- abs(aaclass$CLASS_MAP-aaclass$CLASS_AA) # adds a column in aaclass of absolute predicted - observed values
aaclass$errp_o <- aaclass$CLASS_MAP-aaclass$CLASS_AA # add a column in aaclass of predicted - observed values
levels <- c(-Inf,9.99999,49.99999, 89.99999, Inf) #defines levels that data will be divided at
labels <- c("<10","=>10<50",">=50<90", ">=90") #defines lables that are going to be used in the table
aaclass$classes <-cut(aaclass$CLASS_MAP, levels, labels, right=TRUE)
head(boxplot(aaclass$errp_o~aaclass$classes)) #produces a boxplot that has divided the errp_o into classes (levels)
mean_errp_o <- aggregate(aaclass[, 6], list(aaclass$classes), mean) #determines mean of values in column 6 (errp_o) based on classes (levels)
med_errp_o <- aggregate(aaclass[, 6], list(aaclass$classes), median)#determines median of values in column 6 (errp_o) based on classes (levels)
sd_errp_o <- aggregate(aaclass[, 6], list(aaclass$classes), sd)#determines standard deviation of values in column 6 (errp_o) based on classes (levels)
n_errp_o <- aggregate(aaclass[, 6], list(aaclass$classes), length)#determines number of replicates in column 6 (errp_o) based on classes (levels)
summary_aaclass <-data.frame(c(n_errp_o,mean_errp_o, med_errp_o, sd_errp_o))#creates a table with statistics
summary_aaclass <- data.frame(summary_aaclass[,1], summary_aaclass[,2], summary_aaclass[,4], summary_aaclass[,6], summary_aaclass[,8])
head(summary_aaclass)
colnames(summary_aaclass)[1]<-"Classes"
colnames(summary_aaclass)[2]<-"N"
colnames(summary_aaclass)[3]<-"Mean_error"
colnames(summary_aaclass)[4]<-"Med_error"
colnames(summary_aaclass)[5]<-"SD_error"
summary_aaclass$SE_error <- summary_aaclass$SD_error/sqrt(summary_aaclass$N)
head(summary_aaclass)
write.table(summary_aaclass, file = (paste(path2aatable,sep="")), row.names=FALSE, col.names=TRUE, sep=",")


####plotting obs vs predicted values ####

aaclass$CLASS_MAP_dum <- aaclass$CLASS_MAP #produces a dummy variable of predicted values
aaclass$Err <- aaclass$CLASS_AA-aaclass$CLASS_MAP #produces the error values observed - predicted
fit <- lm(CLASS_MAP_dum~CLASS_MAP, data = aaclass) #fits a linear model using Predicted values and 
aaclass$predicted <- predict(fit) #adds "predicted" values to d so that one can produce a 1:1 data points of predicted values

pl <- ggplot(aaclass, aes(x=CLASS_MAP, y=CLASS_AA)) +
  geom_smooth(method = 'lm', se= TRUE, colour = "black") + #adds line of best fit (black) and 95% confidence interval (grey)
  geom_segment(aes(xend = CLASS_MAP, yend = predicted), alpha = .2) + #adds lines from the 1:1 predicted points to the data points
  geom_point(aes(fill = Err), size = 3, shape = 21) +
  scale_fill_gradient2(name = "Obs - Pred (%)",low = "blue", mid = "white", high = "red") + #provides legend name and colour scale for fill points
  geom_point(aes(y= predicted), shape = 16)  + #adds the 1:1 data points shape 16 is a filled black circle 
  geom_line(aes(y=predicted, x=CLASS_MAP), linetype = 2) + #adds a dotted line between 1:1 points
  theme_bw() +#changes the theme to black and white
  scale_y_continuous(breaks=seq(0,100,10)) + # changes y-axis values to every 10 units
  scale_x_continuous(breaks=seq(0,100,10)) + # changes x-axis values to every 10 units
  theme(panel.grid.minor = element_blank()) + #removes minor grid lines
  theme(axis.title = element_text(size=7, face = 'bold')) + # changes size of x and y axis title text
  theme(axis.text = element_text(size=7)) + # changes size of x and y axis values text
  theme(legend.text = element_text(size=7)) + # changes size of text in legend
  theme(legend.title = element_text(size=7, face = 'bold')) + #changes size of legend titles
  xlab("Predicted coverage (%)") + #adds x-axis label #changes name of x axis title
  ylab("Observed coverage (%)") #adds y-axis label #changes name of y axis title

pl

ggsave(filename = paste(path2aaplot,sep=""), width = 15, height = 9.8, units = 'cm', device = 'pdf', useDingbats=FALSE)

####Comparisons of regression slope against 1:1 line ####

model <- lm(CLASS_AA ~ CLASS_MAP, data = aaclass)
sfit <- summary(model)
sfit
# Calculates two tailed probability for sfit slope against 1:1 line
tstats <- (1-sfit$coefficients[2,1])/sfit$coefficients[2,2]
pval<- 2 * pt(abs(tstats), df = df.residual(model), lower.tail = FALSE)
print(pval)
linearHypothesis(model, "CLASS_MAP = 1") # simpler way of calcultating  sfit slope
y_int <- (sfit$coefficients[1,1]) #y-intercept of sfit
slope <- (sfit$coefficients[2,1]) #slope of sfit
r2 <- sfit$r.squared #r2 value for sfit
y_int_p <- (sfit$coefficients[1,4]) #significance value (p) for sfit y-intercept against y-intercept = 0 (i.e. 1:1 line)
slope_p <- (sfit$coefficients[2,4])
slope_p_1_1 <- pval #significance value (p) for sfit slope against slope = 1 (i.e. 1:1 line)


regression_aaclass <- data.frame(y_int,slope,r2,y_int_p, slope_p, slope_p_1_1) #converts to table
regression_aaclass
write.table(regression_aaclass, file = (paste(path2aaregtable,sep="")), row.names=FALSE, col.names=TRUE, sep=",") #saves table as .csv

#### Theil's partial inequality coefficient ####
SSPE <- sum((aaclass$CLASS_AA-aaclass$CLASS_MAP)^2) # Squared sum of predictive error
U_bias <- (154*((mean(aaclass$CLASS_AA-aaclass$CLASS_MAP))^2))/SSPE #proportion associated mean differences between obs and pred values
U_slope <- ((sfit$coefficients[2,1]-1)^2*sum((aaclass$CLASS_MAP-mean(aaclass$CLASS_MAP))^2))/SSPE #proportion associated with slope of fitted model and 1:1 line
U_unexp <- (sum((residuals.lm(model)^2)))/SSPE #proportion associated with unexplained variance
U_bias + U_slope + U_unexp # should equal 1
theil_aaclass <- data.frame(SSPE, U_bias, U_slope, U_unexp)
theil_aaclass
write.table(theil_aaclass, file = (paste(path2aatheiltable,sep="")), row.names=FALSE, col.names=TRUE, sep=",") #saves table as .csv
